@@ -10,41 +10,40 @@
 #include "evilwm.h"
 #include "log.h"
 
-Display		*dpy;
-int 		num_screens;
-ScreenInfo	*screens;
-ScreenInfo	*current_screen;
-Client		*current = NULL;
-volatile Window	initialising = None;
-XFontStruct	*font;
-Client		*head_client = NULL;
-Application	*head_app = NULL;
-Atom		xa_wm_state;
-Atom		xa_wm_change_state;
-Atom		xa_wm_protos;
-Atom		xa_wm_delete;
-Atom		xa_wm_cmapwins;
-Cursor		move_curs;
-Cursor		resize_curs;
-const char	*opt_display = "";
-const char	*opt_font = DEF_FONT;
-const char	*opt_fg = DEF_FG;
-const char	*opt_bg = DEF_BG;
-const char	*opt_term[3] = { DEF_TERM, DEF_TERM, NULL };
-int		opt_bw = DEF_BW;
+Display         *dpy;
+int             num_screens;
+ScreenInfo      *screens;
+ScreenInfo      *current_screen;
+Client          *current = NULL;
+volatile Window initialising = None;
+XFontStruct     *font;
+Client          *head_client = NULL;
+Application     *head_app = NULL;
+Atom            xa_wm_state;
+Atom            xa_wm_change_state;
+Atom            xa_wm_protos;
+Atom            xa_wm_delete;
+Atom            xa_wm_cmapwins;
+Cursor          move_curs;
+Cursor          resize_curs;
+const char      *opt_display = "";
+const char      *opt_font = DEF_FONT;
+const char      *opt_fg = DEF_FG;
+const char      *opt_bg = DEF_BG;
+const char      *opt_term[3] = { DEF_TERM, DEF_TERM, NULL };
+int             opt_bw = DEF_BW;
 #ifdef VWM
-const char	*opt_fc = DEF_FC;
-int		vdesk = KEY_TO_VDESK(XK_1);
+const char      *opt_fc = DEF_FC;
+int             vdesk = KEY_TO_VDESK(XK_1);
 #endif
 #ifdef SNAP
-int		opt_snap = 0;
+int             opt_snap = 0;
 #endif
 #ifdef SHAPE
-int		have_shape, shape_event;
+int             have_shape, shape_event;
 #endif
-int		quitting = 0;
 #ifdef MWM_HINTS
-Atom		mwm_hints;
+Atom            mwm_hints;
 #endif
 unsigned int numlockmask = 0;
 static unsigned int grabmask1 = ControlMask|Mod1Mask;
@@ -129,9 +128,17 @@ int main(int argc, char *argv[]) {
 			exit(0);
 #endif
 		} else {
-			LOG_INFO("usage: evilwm [-display display] [-term termprog] [-fg foreground]\n");
-			LOG_INFO("\t[-bg background] [-bw borderwidth] [-snap num] [-V]\n");
-			exit(2);
+			LOG_INFO("usage: evilwm [-display display] [-term termprog] [-fg foreground]");
+#ifdef VWM
+			LOG_INFO(" [-fc fixed]");
+#endif
+			LOG_INFO("\n              [-bg background] [-bw borderwidth] [-snap num]\n");
+			LOG_INFO("              [-mask1 modifiers] [-mask2 modifiers]");
+#ifdef VWM
+			LOG_INFO("\n              [-app name/class] [-g geometry] [-v vdesk]");
+#endif
+			LOG_INFO(" [-V]\n");
+			exit((!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help"))?0:1);
 		}
 	}
 
@@ -193,6 +200,7 @@ static void setup_display(void) {
 		exit(1);
 	}
 	XSetErrorHandler(handle_xerror);
+	/* XSynchronize(dpy, True); */
 
 	xa_wm_state = XInternAtom(dpy, "WM_STATE", False);
 	xa_wm_change_state = XInternAtom(dpy, "WM_CHANGE_STATE", False);
@@ -242,7 +250,10 @@ static void setup_display(void) {
 
 	/* SHAPE extension? */
 #ifdef SHAPE
-	have_shape = XShapeQueryExtension(dpy, &shape_event, &i);
+	{
+		int e_dummy;
+		have_shape = XShapeQueryExtension(dpy, &shape_event, &e_dummy);
+	}
 #endif
 
 	/* now set up each screen in turn */
@@ -281,13 +292,11 @@ static void setup_display(void) {
 		screens[i].invert_gc = XCreateGC(dpy, screens[i].root, GCFunction | GCSubwindowMode | GCLineWidth | GCFont, &gv);
 
 		XChangeWindowAttributes(dpy, screens[i].root, CWEventMask, &attr);
-		/* Unfortunately grabbing AnyKey under Solaris seems not to work */
-		/* XGrabKey(dpy, AnyKey, ControlMask|Mod1Mask, root, True, GrabModeAsync, GrabModeAsync); */
-		/* So now I grab each and every one. */
-
+		/* Grab key combinations we're interested in */
 		for (keysym = keys_to_grab; *keysym; keysym++) {
 			grab_keysym(screens[i].root, grabmask1, *keysym);
 		}
+		grab_keysym(screens[i].root, grabmask1 ^ ShiftMask, KEY_KILL);
 		grab_keysym(screens[i].root, grabmask2, KEY_NEXT);
 
 		/* scan all the windows on this screen */
