@@ -40,6 +40,7 @@ void make_new_client(Window w, ScreenInfo *s) {
 	 * trying to manage it. */
 	initialising = w;
 	XFetchName(dpy, w, &name);
+	XSync(dpy, False);
 	/* If 'initialising' is now set to None, that means doing the
 	 * XFetchName raised BadWindow - the window has been removed before
 	 * we got a chance to grab the server. */
@@ -64,6 +65,7 @@ void make_new_client(Window w, ScreenInfo *s) {
 	c->screen = s;
 	c->window = w;
 	c->ignore_unmap = 0;
+	c->remove = 0;
 
 	c->border = opt_bw;
 #ifdef MWM_HINTS
@@ -166,10 +168,9 @@ void make_new_client(Window w, ScreenInfo *s) {
 		XFree(class);
 	}
 
-	//send_config(c);
+	/* send_config(c); */
 #ifndef MOUSE
-	setmouse(c->window, c->width + c->border - 1,
-			c->height + c->border - 1);
+	set_mouse_corner(c);
 #endif
 }
 
@@ -197,7 +198,7 @@ static void init_geometry(Client *c) {
 	size_flags = get_wm_normal_hints(c);
 
 	if ((attr.width >= c->min_width) && (attr.height >= c->min_height)) {
-	//if (attr.map_state == IsViewable || (size_flags & (PSize | USSize))) {
+	/* if (attr.map_state == IsViewable || (size_flags & (PSize | USSize))) { */
 		c->width = attr.width;
 		c->height = attr.height;
 	} else {
